from typing import BinaryIO
import obspy
from record_reader import RecordReader


class MSeedReader:
    def __init__(self, file: BinaryIO) -> None:
        self.file = file

    def read(self):
        trace = []
        while 1:
            # We know that end of file is reached when read() returns an empty
            # string.
            is_eof = (self.file.read(1) == b'')
            if is_eof:
                break
            # We need to move the file pointer back to its position after we
            # do the end of file check. Otherwise, we would be off by one
            # byte for all the reads afterward.
            self.file.seek(-1, 1)

            # We save the start of the current record so that after we are
            # done reading the record, we can move back. This makes moving
            # to the next record a lot easier, seeing as we can simply move
            # the file pointer a distance the size of the current record.
            current_record_start = self.file.tell()

            reader = RecordReader(self.file)
            trace.append(reader.get_first_data_point())
            # sample_count = reader.record_metadata.sample_count
            # sample_rate = reader.record_metadata.sample_rate
            # record_time_taken = sample_count / sample_rate
            # record_end_time = (reader.record_metadata.start_time +
            #                    record_time_taken)

            # MSEED stores the size of a data record as an exponent of a
            # power of two, so we have to convert that to actual size before
            # doing anything else.
            record_length_exp = reader.header_unpacker.unpack(
                'B', reader.blockette_1000.record_length
            )[0]
            record_size = 2 ** record_length_exp

            self.file.seek(current_record_start)
            self.file.seek(record_size, 1)


if __name__ == '__main__':
    # numpy.set_printoptions(threshold=sys.maxsize)
    file_path = '/Users/ldam/Documents/GIT/sohstationviewer/tests/test_data/' \
                'Q330_mixed_traces/XX-3203_4-20221222183011'
    file = open(file_path, 'rb')
    stream = obspy.read(file_path)
    MSeedReader(file).read()
